package ilog.rules.support.debugtrace;

import ilog.rules.res.model.trace.IlrDWTrace;
import ilog.rules.res.persistence.impl.jdbc.IlrDAOConfigurator;
import ilog.rules.res.persistence.impl.jdbc.IlrJDBCConnectionProvider;
import ilog.rules.res.persistence.trace.IlrTraceDAOException;
import ilog.rules.res.persistence.trace.impl.jdbc.IlrGenericTraceDAO;
import ilog.rules.res.session.IlrSessionRequest;
import ilog.rules.res.session.IlrSessionResponse;
import ilog.rules.res.session.IlrWarning;
import ilog.rules.res.session.ruleset.IlrExecutionEvent;
import ilog.rules.res.session.ruleset.IlrTaskEvent;
import ilog.rules.res.session.util.IlrXMLExecutionTraceSerializer;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class DebugTraceDAO extends IlrGenericTraceDAO {
	static final Logger LOG = Logger.getLogger(DebugTraceDAO.class.getName());
	static final String NL = System.getProperty("line.separator");
	static final String INDENT_INCREMENT = "  ";

	public DebugTraceDAO(IlrDAOConfigurator configurator, IlrJDBCConnectionProvider connectionProvider) {
		super(configurator, connectionProvider);
	}

	public void saveTrace(IlrDWTrace trace, IlrSessionRequest request, IlrSessionResponse response)
			throws IlrTraceDAOException {
		try {
			logTrace(trace, request, response);
		} catch (Throwable t){
			String execId = trace.getExecutionId();
			LOG.log(Level.SEVERE, execId + " logging save trace error", t);
		}
		try {
			super.saveTrace(trace, request, response);
		} catch (IlrTraceDAOException tde) {
			String execId = trace.getExecutionId();
			LOG.log(Level.SEVERE, execId + " save trace error", tde);
			throw tde;
		} catch (Throwable t){
			String execId = trace.getExecutionId();
			LOG.log(Level.SEVERE, execId + " save trace error", t);
		}
	}

	protected void logTrace(IlrDWTrace trace, IlrSessionRequest request, IlrSessionResponse response) {
		if (LOG.isLoggable(Level.INFO)) {
			StringBuffer sb = new StringBuffer();
			sb.append(NL + "ruleset execution id: ");
			sb.append(trace.getExecutionId() + NL);
			sb.append("executed ruleset path: ");
			sb.append(trace.getExecutedRulesetPath() + NL);
			sb.append("execution duration: ");
			sb.append(trace.getExecutionDuration());
			sb.append("ms" + NL);
			sb.append("total rules fired: ");
			sb.append(trace.getTotalRulesFired() + NL);
			String fullXmlTrace = null;
			try {
				fullXmlTrace = new IlrXMLExecutionTraceSerializer(trace).getXMLString();
			} catch (Exception e) {
				LOG.log(Level.INFO, "error serializing trace to XML", e);
			}
			if (fullXmlTrace != null) {
				sb.append("full execution trace length: ");
				sb.append(fullXmlTrace.length() + NL);
			}
			logExecutionEvents(trace.getExecutionEvents(), sb, "> ");
			LOG.info(sb.toString());
			if (LOG.isLoggable(Level.FINEST)) {
				String execId = trace.getExecutionId();
				for (IlrWarning warning : response.getWarnings()) {
					String msg = execId + " session response warning : " + warning.getMessage();
					LOG.log(Level.FINEST, msg, warning.getErrorCause());
				}
				StringBuffer sb2 = new StringBuffer();
				sb2.append(NL + "ruleset execution id: ");
				sb2.append(trace.getExecutionId() + NL);
				sb2.append(fullXmlTrace == null ? "NO XML TRACE" : fullXmlTrace);
				LOG.finest(sb2.toString());
			}
		}
	}

	protected void logExecutionEvents(List<IlrExecutionEvent> eventList, StringBuffer sb, String indent) {
		for (IlrExecutionEvent event : eventList) {
			sb.append(indent + event.getName() + NL);
			if (event instanceof IlrTaskEvent) {
				logExecutionEvents(((IlrTaskEvent) event).getSubExecutionEvents(), sb, indent + INDENT_INCREMENT);
			}
		}
	}
}
